#include "csapp.h"
#include "linked_list.h"

llist_t list;

void initjobs()
{
  list.head = NULL;
  list.node_count = 0;
  pthread_mutex_init(&(list.mutex), NULL);
}
sigset_t mask;

void lock(){
	Sigemptyset(&mask);
	Sigaddset(&mask, SIGCHLD); 
	Sigprocmask(SIG_BLOCK, &mask, NULL); /* Block SIGCHLD */
}

void unlock(){
    Sigprocmask(SIG_UNBLOCK, &mask, NULL); /* Unblock SIGCHLD */
}

void addjob(int pid)
{
    lock();
    // Create a new node
    // This has to be done on the heap!
    node_t *node = (node_t*)malloc(sizeof(node_t));

    // MUST initialize data before use!
    node->next = NULL;
    node->data = pid;

    int res = add_front(&list, node);
    if (res != 0) {
      printf("Error adding node! Error code: %d\n", res);
      return;
    }
    unlock();
}

void deletejob(int pid)
{
    list.head = delete(&list,pid);
    pthread_mutex_unlock(&(list.mutex));
}

/* $begin procmask1 */
void handler(int sig)
{
    pid_t pid;
    
    while ((pid = waitpid(-1, NULL, 0)) > 0) /* Reap a zombie child */
    	deletejob(pid); /* Delete the child from the job list */
    
    if (errno != ECHILD)
	    unix_error("waitpid error");
}
    
int main(int argc, char **argv)
{
    int pid;

    Signal(SIGCHLD, handler);
    initjobs(); /* Initialize the job list */

    int i = 100;
    while (i--) {
	/* Child process */
	if ((pid = Fork()) == 0) {
	    Execve("/bin/date", argv, NULL);
	}

	/* Parent process */
	addjob(pid);  /* Add the child to the job list */
    }

    //print the list of jobs still not deleted
    
    printf("Printing the list of jobs still not deleted\n"); 

    while(list.node_count){
        printf("Waiting for %d remaining jobs to complete\n",list.node_count);
        print_list(&list);
        sleep(1);
    }


    exit(0);
}
/* $end procmask1 */
